@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2020 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2021 Roger Light <roger@atchoo.org>
 
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License 2.0
@@ -33,7 +33,6 @@ Contributors:
 #endif
 
 #include <mosquitto.h>
-#include <mqtt_protocol.h>
 #include "client_shared.h"
 #include "pub_shared.h"
 #include "sub_client_output.h"
@@ -44,7 +43,7 @@ enum rr__state {
 	rr_s_subscribed,
 	rr_s_ready_to_publish,
 	rr_s_wait_for_response,
-	rr_s_disconnect
+	rr_s_disconnect,
 };
 
 static enum rr__state client_state = rr_s_new;
@@ -54,8 +53,12 @@ int msg_count = 0;
 struct mosquitto *g_mosq = NULL;
 static bool timed_out = false;
 static int connack_result = 0;
+static struct timespec publish_send_time;
+static struct timespec publish_recv_time;
 
 #ifndef WIN32
+
+
 static void my_signal_handler(int signum)
 {
 	if(signum == SIGALRM){
@@ -69,6 +72,8 @@ static void my_signal_handler(int signum)
 
 int my_publish(struct mosquitto *mosq, int *mid, const char *topic, int payloadlen, void *payload, int qos, bool retain)
 {
+	mosquitto_time_ns(&publish_send_time.tv_sec, &publish_send_time.tv_nsec);
+
 	if(cfg.protocol_version < MQTT_PROTOCOL_V5){
 		return mosquitto_publish_v5(mosq, mid, topic, payloadlen, payload, qos, retain, NULL);
 	}else{
@@ -83,8 +88,14 @@ static void my_message_callback(struct mosquitto *mosq, void *obj, const struct
 	UNUSED(obj);
 	UNUSED(properties);
 
-	if(process_messages == false) return;
-	if(message->retain && cfg.no_retain) return;
+	if(process_messages == false){
+		return;
+	}
+	if(message->retain && cfg.no_retain){
+		return;
+	}
+
+	mosquitto_time_ns(&publish_recv_time.tv_sec, &publish_recv_time.tv_nsec);
 
 	print_message(&cfg, message, properties);
 
@@ -95,12 +106,14 @@ static void my_message_callback(struct mosquitto *mosq, void *obj, const struct
 		case MSGMODE_NULL:
 			client_state = rr_s_disconnect;
 			break;
+
 		case MSGMODE_STDIN_LINE:
 			client_state = rr_s_ready_to_publish;
 			break;
 	}
 }
 
+
 void my_connect_callback(struct mosquitto *mosq, void *obj, int result, int flags, const mosquitto_property *properties)
 {
 	UNUSED(obj);
@@ -110,15 +123,11 @@ void my_connect_callback(struct mosquitto *mosq, void *obj, int result, int flag
 	connack_result = result;
 	if(!result){
 		client_state = rr_s_connected;
-		mosquitto_subscribe_v5(mosq, NULL, cfg.response_topic, cfg.qos, 0, cfg.subscribe_props);
+		mosquitto_subscribe_v5(mosq, NULL, cfg.response_topic, cfg.qos, cfg.sub_opts, cfg.subscribe_props);
 	}else{
 		client_state = rr_s_disconnect;
 		if(result){
-			if(result == MQTT_RC_UNSUPPORTED_PROTOCOL_VERSION){
-				err_printf(&cfg, "Connection error: %s. mosquitto_rr only supports connecting to an MQTT v5 broker\n", mosquitto_reason_string(result));
-			}else{
-				err_printf(&cfg, "Connection error: %s\n", mosquitto_reason_string(result));
-			}
+			err_printf(&cfg, "Connection error: %s\n", mosquitto_reason_string(result));
 		}
 		mosquitto_disconnect_v5(mosq, 0, cfg.disconnect_props);
 	}
@@ -141,18 +150,6 @@ static void my_subscribe_callback(struct mosquitto *mosq, void *obj, int mid, in
 }
 
 
-void my_publish_callback(struct mosquitto *mosq, void *obj, int mid, int reason_code, const mosquitto_property *properties)
-{
-	UNUSED(mosq);
-	UNUSED(obj);
-	UNUSED(mid);
-	UNUSED(reason_code);
-	UNUSED(properties);
-
-	client_state = rr_s_wait_for_response;
-}
-
-
 static void print_version(void)
 {
 	int major, minor, revision;
@@ -161,6 +158,7 @@ static void print_version(void)
 	printf("mosquitto_rr version %s running on libmosquitto %d.%d.%d.\n", VERSION, major, minor, revision);
 }
 
+
 static void print_usage(void)
 {
 	int major, minor, revision;
@@ -171,7 +169,8 @@ static void print_usage(void)
 	printf("             with v3.1.1 brokers.\n");
 	printf("mosquitto_rr version %s running on libmosquitto %d.%d.%d.\n\n", VERSION, major, minor, revision);
 	printf("Usage: mosquitto_rr {[-h host] [--unix path] [-p port] [-u username] [-P password] -t topic | -L URL} -e response-topic\n");
-	printf("                    [-c] [-k keepalive] [-q qos] [-R] [-x session-expiry-interval\n");
+	printf("                    {-f file | -l | -n | -m message}\n");
+	printf("                    [-c] [-k keepalive] [-q qos] [-R] [-x session-expiry-interval]\n");
 	printf("                    [-F format]\n");
 #ifndef WIN32
 	printf("                    [-W timeout_secs]\n");
@@ -185,6 +184,7 @@ static void print_usage(void)
 	printf("                    [-d] [-N] [--quiet] [-v]\n");
 	printf("                    [--will-topic [--will-payload payload] [--will-qos qos] [--will-retain]]\n");
 #ifdef WITH_TLS
+	printf("                    [--no-tls]\n");
 	printf("                    [{--cafile file | --capath dir} [--cert file] [--key file]\n");
 	printf("                      [--ciphers ciphers] [--insecure]\n");
 	printf("                      [--tls-alpn protocol]\n");
@@ -198,6 +198,7 @@ static void print_usage(void)
 	printf("                    [--proxy socks-url]\n");
 #endif
 	printf("                    [-D command identifier value]\n");
+	printf("                    [-o options-file]\n");
 	printf("       mosquitto_rr --help\n\n");
 	printf(" -A : bind the outgoing socket to this host/ip address. Use to control which interface\n");
 	printf("      the client communicates over.\n");
@@ -215,7 +216,10 @@ static void print_usage(void)
 	printf(" -k : keep alive in seconds for this client. Defaults to 60.\n");
 	printf(" -L : specify user, password, hostname, port and topic as a URL in the form:\n");
 	printf("      mqtt(s)://[username[:password]@]host[:port]/topic\n");
+	printf("      ws(s)://[username[:password]@]host[:port]/topic\n");
 	printf(" -N : do not add an end of line character when printing the payload.\n");
+	printf(" -o : provide options in a file rather than on the command line.\n");
+	printf("      See the Options section of https://mosquitto.org/man/mosquitto_pub-1.html\n");
 	printf(" -p : network port to connect to. Defaults to 1883 for plain MQTT and 8883 for MQTT over TLS.\n");
 	printf(" -P : provide a password\n");
 	printf(" -q : quality of service level to use for communications. Defaults to 0.\n");
@@ -249,6 +253,7 @@ static void print_usage(void)
 	printf(" --will-qos : QoS level for the client Will.\n");
 	printf(" --will-retain : if given, make the client Will retained.\n");
 	printf(" --will-topic : the topic on which to publish the client Will.\n");
+	printf(" --ws : connect using WebSockets.\n");
 #ifdef WITH_TLS
 	printf(" --cafile : path to a file containing trusted CA certificates to enable encrypted\n");
 	printf("            certificate based communication.\n");
@@ -258,11 +263,11 @@ static void print_usage(void)
 	printf(" --key : client private key for authentication, if required by server.\n");
 	printf(" --ciphers : openssl compatible list of TLS ciphers to support.\n");
 	printf(" --tls-use-os-certs : Load and trust OS provided CA certificates.\n");
-	printf(" --tls-version : TLS protocol version, can be one of tlsv1.3 tlsv1.2 or tlsv1.1.\n");
+	printf(" --tls-version : TLS protocol version, can be one of tlsv1.3 or tlsv1.2.\n");
 	printf("                 Defaults to tlsv1.2 if available.\n");
-	printf(" --insecure : do not check that the server certificate hostname matches the remote\n");
-	printf("              hostname. Using this option means that you cannot be sure that the\n");
-	printf("              remote host is the server you wish to connect to and so is insecure.\n");
+	printf(" --insecure : do not verify the the server certificate. Using this option means that\n");
+	printf("              you cannot be sure that the remote host is the server you wish to connect\n");
+	printf("              to and so is insecure.\n");
 	printf("              Do not use this option in a production environment.\n");
 #ifdef WITH_TLS_PSK
 	printf(" --psk : pre-shared-key in hexadecimal (no leading 0x) to enable TLS-PSK mode.\n");
@@ -277,15 +282,41 @@ static void print_usage(void)
 	printf("\nSee https://mosquitto.org/ for more information.\n\n");
 }
 
+
+static void report_latency(void)
+{
+	if(cfg.measure_latency){
+		time_t s = publish_recv_time.tv_sec - publish_send_time.tv_sec;
+		long ns = publish_recv_time.tv_nsec - publish_send_time.tv_nsec;
+
+		if(ns < 0){
+			s--;
+			ns += 1000000000;
+		}
+
+		if(s > 0){
+			printf("Latency: %ld.%09ld\n", s, ns);
+		}else{
+			if(ns < 1000){
+				printf("Latency: %ldns\n", ns);
+			}else if(ns < 1000000){
+				printf("Latency: %fµs\n", ((double)ns)/1000.0);
+			}else{
+				printf("Latency: %fms\n", ((double)ns)/1000000.0);
+			}
+		}
+	}
+}
+
+
 int main(int argc, char *argv[])
 {
 	int rc;
 #ifndef WIN32
-		struct sigaction sigact;
+	struct sigaction sigact;
 #endif
 
 	mosquitto_lib_init();
-	output_init();
 
 	rc = client_config_load(&cfg, CLIENT_RR, argc, argv);
 	if(rc){
@@ -301,6 +332,18 @@ int main(int argc, char *argv[])
 		goto cleanup;
 	}
 
+	if(cfg.pub_mode == MSGMODE_STDIN_FILE){
+		if(load_stdin()){
+			err_printf(&cfg, "Error loading input from stdin.\n");
+			goto cleanup;
+		}
+	}else if(cfg.file_input){
+		if(load_file(cfg.file_input)){
+			err_printf(&cfg, "Error loading input file \"%s\".\n", cfg.file_input);
+			goto cleanup;
+		}
+	}
+
 	if(!cfg.topic || cfg.pub_mode == MSGMODE_NONE || !cfg.response_topic){
 		fprintf(stderr, "Error: All of topic, message, and response topic must be supplied.\n");
 		fprintf(stderr, "\nUse 'mosquitto_rr --help' to see usage.\n");
@@ -316,8 +359,9 @@ int main(int argc, char *argv[])
 		err_printf(&cfg, "Error in PUBLISH properties: Duplicate response topic.\n");
 		goto cleanup;
 	}
+	output_init(&cfg);
 
-	if(client_id_generate(&cfg)){
+	if(clientid_generate(&cfg)){
 		goto cleanup;
 	}
 
@@ -383,6 +427,8 @@ int main(int argc, char *argv[])
 		}
 	}while(rc == MOSQ_ERR_SUCCESS && client_state != rr_s_disconnect);
 
+	report_latency();
+
 	mosquitto_destroy(g_mosq);
 	mosquitto_lib_cleanup();
 
