@@ -6,21 +6,14 @@
 
 #ifdef __APPLE__
 #  define __DARWIN_C_SOURCE
-#elif defined(__FreeBSD__) || defined(__NetBSD__) || defined(__SYMBIAN32__)
+#elif defined(__FreeBSD__) || defined(__NetBSD__)
 #  define HAVE_NETINET_IN_H
 #elif defined(__QNX__)
 #  define _XOPEN_SOURCE 600
 #  define __BSD_VISIBLE 1
 #  define HAVE_NETINET_IN_H
-#else
-#  define _XOPEN_SOURCE 700
-#  define _DEFAULT_SOURCE 1
-#  define _POSIX_C_SOURCE 200809L
-#endif
-
-
-#ifndef _GNU_SOURCE
-#  define _GNU_SOURCE
+#elif defined(_AIX)
+#  define HAVE_NETINET_IN_H
 #endif
 
 #define OPENSSL_LOAD_CONF
@@ -43,16 +36,20 @@
 #endif
 
 #ifdef WIN32
-#  ifndef strcasecmp
-#    define strcasecmp strcmpi
-#  endif
+#  define strcasecmp _stricmp
+#  define strncasecmp _strnicmp
 #  define strtok_r strtok_s
 #  define strerror_r(e, b, l) strerror_s(b, l, e)
+
+#  ifdef _MSC_VER
+#    include <basetsd.h>
+typedef SSIZE_T ssize_t;
+#  endif
 #endif
 
 
 #define uthash_malloc(sz) mosquitto_malloc(sz)
-#define uthash_free(ptr,sz) mosquitto_free(ptr)
+#define uthash_free(ptr, sz) mosquitto_free(ptr)
 
 
 #ifdef WITH_TLS
@@ -76,13 +73,26 @@
 #define UNUSED(A) (void)(A)
 
 /* Android Bionic libpthread implementation doesn't have pthread_cancel */
-#ifndef ANDROID
+#if !defined(ANDROID) && !defined(WIN32)
 #  define HAVE_PTHREAD_CANCEL
 #endif
 
-#ifdef WITH_CJSON
-#  include <cjson/cJSON.h>
-#  define CJSON_VERSION_FULL (CJSON_VERSION_MAJOR*1000000+CJSON_VERSION_MINOR*1000+CJSON_VERSION_PATCH)
+#define WS_IS_LWS 1
+#define WS_IS_BUILTIN 2
+
+#ifdef WITH_BROKER
+#  ifdef __GNUC__
+#    define BROKER_EXPORT __attribute__((__used__))
+#  else
+#    define BROKER_EXPORT
+#  endif
+#else
+#  define BROKER_EXPORT
 #endif
 
+#define TOPIC_HIERARCHY_LIMIT 200
+
+#ifdef WITH_ADNS
+#  define _GNU_SOURCE
+#endif
 #endif
