@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2020 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2021 Roger Light <roger@atchoo.org>
 
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License 2.0
@@ -24,11 +24,11 @@ Contributors:
 #  include "mosquitto_broker_internal.h"
 #endif
 
+#include "callbacks.h"
 #include "mosquitto.h"
 #include "mosquitto_internal.h"
 #include "logging_mosq.h"
-#include "memory_mosq.h"
-#include "mqtt_protocol.h"
+#include "mosquitto/mqtt_protocol.h"
 #include "packet_mosq.h"
 #include "property_mosq.h"
 #include "read_handle.h"
@@ -48,6 +48,9 @@ int handle__suback(struct mosquitto *mosq)
 	assert(mosq);
 
 	if(mosquitto__get_state(mosq) != mosq_cs_active){
+#ifdef WITH_BROKER
+		log__printf(NULL, MOSQ_LOG_INFO, "Protocol error from %s: SUBACK before session is active.", mosq->id);
+#endif
 		return MOSQ_ERR_PROTOCOL;
 	}
 	if(mosq->in_packet.command != CMD_SUBACK){
@@ -57,6 +60,7 @@ int handle__suback(struct mosquitto *mosq)
 #ifdef WITH_BROKER
 	if(mosq->bridge == NULL){
 		/* Client is not a bridge, so shouldn't be sending SUBACK */
+		log__printf(NULL, MOSQ_LOG_INFO, "Protocol error from %s: SUBACK when not a bridge.", mosq->id);
 		return MOSQ_ERR_PROTOCOL;
 	}
 	log__printf(NULL, MOSQ_LOG_DEBUG, "Received SUBACK from %s", SAFE_PRINT(mosq->id));
@@ -64,30 +68,35 @@ int handle__suback(struct mosquitto *mosq)
 	log__printf(mosq, MOSQ_LOG_DEBUG, "Client %s received SUBACK", SAFE_PRINT(mosq->id));
 #endif
 	rc = packet__read_uint16(&mosq->in_packet, &mid);
-	if(rc) return rc;
-	if(mid == 0) return MOSQ_ERR_PROTOCOL;
+	if(rc){
+		return rc;
+	}
+	if(mid == 0){
+		return MOSQ_ERR_PROTOCOL;
+	}
 
 	if(mosq->protocol == mosq_p_mqtt5){
 		rc = property__read_all(CMD_SUBACK, &mosq->in_packet, &properties);
-		if(rc) return rc;
+		if(rc){
+			return rc;
+		}
 	}
 
 	qos_count = (int)(mosq->in_packet.remaining_length - mosq->in_packet.pos);
-	if(qos_count == 0) return MOSQ_ERR_PROTOCOL;
-	granted_qos = mosquitto__malloc((size_t)qos_count*sizeof(int));
+	if(qos_count == 0){
+		mosquitto_property_free_all(&properties);
+		return MOSQ_ERR_PROTOCOL;
+	}
+	granted_qos = mosquitto_malloc((size_t)qos_count*sizeof(int));
 	if(!granted_qos){
-#ifdef WITH_BROKER
 		mosquitto_property_free_all(&properties);
-#endif
 		return MOSQ_ERR_NOMEM;
 	}
 	while(mosq->in_packet.pos < mosq->in_packet.remaining_length){
 		rc = packet__read_byte(&mosq->in_packet, &qos);
 		if(rc){
-			mosquitto__free(granted_qos);
-#ifdef WITH_BROKER
+			mosquitto_FREE(granted_qos);
 			mosquitto_property_free_all(&properties);
-#endif
 			return rc;
 		}
 		granted_qos[i] = (int)qos;
@@ -97,22 +106,10 @@ int handle__suback(struct mosquitto *mosq)
 	/* Immediately free, we don't do anything with Reason String or User Property at the moment */
 	mosquitto_property_free_all(&properties);
 #else
-	pthread_mutex_lock(&mosq->callback_mutex);
-	if(mosq->on_subscribe){
-		mosq->in_callback = true;
-		mosq->on_subscribe(mosq, mosq->userdata, mid, qos_count, granted_qos);
-		mosq->in_callback = false;
-	}
-	if(mosq->on_subscribe_v5){
-		mosq->in_callback = true;
-		mosq->on_subscribe_v5(mosq, mosq->userdata, mid, qos_count, granted_qos, properties);
-		mosq->in_callback = false;
-	}
-	pthread_mutex_unlock(&mosq->callback_mutex);
+	callback__on_subscribe(mosq, mid, qos_count, granted_qos, properties);
 	mosquitto_property_free_all(&properties);
 #endif
-	mosquitto__free(granted_qos);
+	mosquitto_FREE(granted_qos);
 
 	return MOSQ_ERR_SUCCESS;
 }
-
