@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009-2019 Roger Light <roger@atchoo.org>
+Copyright (c) 2009-2021 Roger Light <roger@atchoo.org>
 
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License 2.0
@@ -26,78 +26,47 @@ Contributors:
 #endif
 
 #ifndef WIN32
-#ifdef WITH_EPOLL
-#include <sys/epoll.h>
-#define MAX_EVENTS 1000
-#endif
-#include <poll.h>
-#include <unistd.h>
-#else
-#include <process.h>
-#include <winsock2.h>
-#include <ws2tcpip.h>
+#  include <sys/epoll.h>
+#  define MAX_EVENTS 1000
 #endif
 
 #include <errno.h>
 #include <signal.h>
-#include <stdio.h>
-#include <string.h>
-#ifndef WIN32
-#  include <sys/socket.h>
-#endif
-#include <time.h>
-
-#ifdef WITH_WEBSOCKETS
-#  include <libwebsockets.h>
-#endif
 
 #include "mosquitto_broker_internal.h"
-#include "memory_mosq.h"
 #include "mux.h"
 #include "packet_mosq.h"
-#include "send_mosq.h"
-#include "sys_tree.h"
-#include "time_mosq.h"
 #include "util_mosq.h"
 
-#ifdef WIN32
-#  error "epoll not supported on WIN32"
-#endif
-
 static void loop_handle_reads_writes(struct mosquitto *context, uint32_t events);
 
-static sigset_t my_sigblock;
 static struct epoll_event ep_events[MAX_EVENTS];
 
-int mux_epoll__init(struct mosquitto__listener_sock *listensock, int listensock_count)
+
+int mux_epoll__init(void)
 {
-	struct epoll_event ev;
-	int i;
-
-#ifndef WIN32
-	sigemptyset(&my_sigblock);
-	sigaddset(&my_sigblock, SIGINT);
-	sigaddset(&my_sigblock, SIGTERM);
-	sigaddset(&my_sigblock, SIGUSR1);
-	sigaddset(&my_sigblock, SIGUSR2);
-	sigaddset(&my_sigblock, SIGHUP);
-#endif
-
 	memset(&ep_events, 0, sizeof(struct epoll_event)*MAX_EVENTS);
 
 	db.epollfd = 0;
-	if ((db.epollfd = epoll_create(MAX_EVENTS)) == -1) {
+	if((db.epollfd = epoll_create(MAX_EVENTS)) == -1){
 		log__printf(NULL, MOSQ_LOG_ERR, "Error in epoll creating: %s", strerror(errno));
 		return MOSQ_ERR_UNKNOWN;
 	}
-	memset(&ev, 0, sizeof(struct epoll_event));
-	for(i=0; i<listensock_count; i++){
+
+	return MOSQ_ERR_SUCCESS;
+}
+
+
+int mux_epoll__add_listeners(struct mosquitto__listener_sock *listensock, int listensock_count)
+{
+	for(int i=0; i<listensock_count; i++){
+		struct epoll_event ev;
+
+		memset(&ev, 0, sizeof(struct epoll_event));
 		ev.data.ptr = &listensock[i];
 		ev.events = EPOLLIN;
-		if (epoll_ctl(db.epollfd, EPOLL_CTL_ADD, listensock[i].sock, &ev) == -1) {
+		if(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, listensock[i].sock, &ev) == -1){
 			log__printf(NULL, MOSQ_LOG_ERR, "Error in epoll initial registering: %s", strerror(errno));
-			(void)close(db.epollfd);
-			db.epollfd = 0;
 			return MOSQ_ERR_UNKNOWN;
 		}
 	}
@@ -105,16 +74,29 @@ int mux_epoll__init(struct mosquitto__listener_sock *listensock, int listensock_
 	return MOSQ_ERR_SUCCESS;
 }
 
+
+int mux_epoll__delete_listeners(struct mosquitto__listener_sock *listensock, int listensock_count)
+{
+	for(int i=0; i<listensock_count; i++){
+		if(epoll_ctl(db.epollfd, EPOLL_CTL_DEL, listensock[i].sock, NULL) == -1){
+			return MOSQ_ERR_UNKNOWN;
+		}
+	}
+
+	return MOSQ_ERR_SUCCESS;
+}
+
+
 int mux_epoll__add_out(struct mosquitto *context)
 {
-	struct epoll_event ev;
+	if(!(context->events & EPOLLOUT)){
+		struct epoll_event ev;
 
-	if(!(context->events & EPOLLOUT)) {
 		memset(&ev, 0, sizeof(struct epoll_event));
 		ev.data.ptr = context;
 		ev.events = EPOLLIN | EPOLLOUT;
-		if(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1) {
-			if((errno != EEXIST)||(epoll_ctl(db.epollfd, EPOLL_CTL_MOD, context->sock, &ev) == -1)) {
+		if(epoll_ctl(db.epollfd, EPOLL_CTL_MOD, context->sock, &ev) == -1){
+			if((errno != ENOENT)||(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1)){
 				log__printf(NULL, MOSQ_LOG_DEBUG, "Error in epoll re-registering to EPOLLOUT: %s", strerror(errno));
 			}
 		}
@@ -126,15 +108,15 @@ int mux_epoll__add_out(struct mosquitto *context)
 
 int mux_epoll__remove_out(struct mosquitto *context)
 {
-	struct epoll_event ev;
+	if(context->events & EPOLLOUT){
+		struct epoll_event ev;
 
-	if(context->events & EPOLLOUT) {
 		memset(&ev, 0, sizeof(struct epoll_event));
 		ev.data.ptr = context;
 		ev.events = EPOLLIN;
-		if(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1) {
-			if((errno != EEXIST)||(epoll_ctl(db.epollfd, EPOLL_CTL_MOD, context->sock, &ev) == -1)) {
-					log__printf(NULL, MOSQ_LOG_DEBUG, "Error in epoll re-registering to EPOLLIN: %s", strerror(errno));
+		if(epoll_ctl(db.epollfd, EPOLL_CTL_MOD, context->sock, &ev) == -1){
+			if((errno != ENOENT)||(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1)){
+				log__printf(NULL, MOSQ_LOG_DEBUG, "Error in epoll re-registering to EPOLLIN: %s", strerror(errno));
 			}
 		}
 		context->events = EPOLLIN;
@@ -143,14 +125,14 @@ int mux_epoll__remove_out(struct mosquitto *context)
 }
 
 
-int mux_epoll__add_in(struct mosquitto *context)
+int mux_epoll__new(struct mosquitto *context)
 {
 	struct epoll_event ev;
 
 	memset(&ev, 0, sizeof(struct epoll_event));
 	ev.events = EPOLLIN;
 	ev.data.ptr = context;
-	if (epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1) {
+	if(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1){
 		if(errno != EEXIST){
 			log__printf(NULL, MOSQ_LOG_ERR, "Error in epoll accepting: %s", strerror(errno));
 		}
@@ -162,11 +144,8 @@ int mux_epoll__add_in(struct mosquitto *context)
 
 int mux_epoll__delete(struct mosquitto *context)
 {
-	struct epoll_event ev;
-
-	memset(&ev, 0, sizeof(struct epoll_event));
 	if(context->sock != INVALID_SOCKET){
-		if(epoll_ctl(db.epollfd, EPOLL_CTL_DEL, context->sock, &ev) == -1){
+		if(epoll_ctl(db.epollfd, EPOLL_CTL_DEL, context->sock, NULL) == -1){
 			return 1;
 		}
 	}
@@ -176,50 +155,48 @@ int mux_epoll__delete(struct mosquitto *context)
 
 int mux_epoll__handle(void)
 {
-	int i;
 	struct epoll_event ev;
-	sigset_t origsig;
 	struct mosquitto *context;
 	struct mosquitto__listener_sock *listensock;
 	int event_count;
 
 	memset(&ev, 0, sizeof(struct epoll_event));
-	sigprocmask(SIG_SETMASK, &my_sigblock, &origsig);
+#if defined(WITH_WEBSOCKETS)
 	event_count = epoll_wait(db.epollfd, ep_events, MAX_EVENTS, 100);
-	sigprocmask(SIG_SETMASK, &origsig, NULL);
+#else
+	event_count = epoll_wait(db.epollfd, ep_events, MAX_EVENTS, db.next_event_ms);
+#endif
 
 	db.now_s = mosquitto_time();
 	db.now_real_s = time(NULL);
 
 	switch(event_count){
-	case -1:
-		if(errno != EINTR){
-			log__printf(NULL, MOSQ_LOG_ERR, "Error in epoll waiting: %s.", strerror(errno));
-		}
-		break;
-	case 0:
-		break;
-	default:
-		for(i=0; i<event_count; i++){
-			context = ep_events[i].data.ptr;
-			if(context->ident == id_client){
-				loop_handle_reads_writes(context, ep_events[i].events);
-			}else if(context->ident == id_listener){
-				listensock = ep_events[i].data.ptr;
+		case -1:
+			if(errno != EINTR){
+				log__printf(NULL, MOSQ_LOG_ERR, "Error in epoll waiting: %s.", strerror(errno));
+			}
+			break;
+		case 0:
+			break;
+		default:
+			for(int i=0; i<event_count; i++){
+				context = ep_events[i].data.ptr;
+				if(context->ident == id_client){
+					loop_handle_reads_writes(context, ep_events[i].events);
+				}else if(context->ident == id_listener){
+					listensock = ep_events[i].data.ptr;
 
-				if (ep_events[i].events & (EPOLLIN | EPOLLPRI)){
-					while((context = net__socket_accept(listensock)) != NULL){
-						context->events = EPOLLIN;
-						mux__add_in(context);
+					if(ep_events[i].events & (EPOLLIN | EPOLLPRI)){
+						while((context = net__socket_accept(listensock)) != NULL){
+						}
 					}
+#if defined(WITH_WEBSOCKETS) && WITH_WEBSOCKETS == WS_IS_LWS
+				}else if(context->ident == id_listener_ws){
+					/* Nothing needs to happen here, because we always call lws_service in the loop.
+					 * The important point is we've been woken up for this listener. */
+#endif
 				}
-#ifdef WITH_WEBSOCKETS
-			}else if(context->ident == id_listener_ws){
-				/* Nothing needs to happen here, because we always call lws_service in the loop.
-				 * The important point is we've been woken up for this listener. */
-#endif
 			}
-		}
 	}
 	return MOSQ_ERR_SUCCESS;
 }
@@ -243,7 +220,7 @@ static void loop_handle_reads_writes(struct mosquitto *context, uint32_t events)
 		return;
 	}
 
-#ifdef WITH_WEBSOCKETS
+#if defined(WITH_WEBSOCKETS) && WITH_WEBSOCKETS == WS_IS_LWS
 	if(context->wsi){
 		struct lws_pollfd wspoll;
 		wspoll.fd = context->sock;
@@ -291,7 +268,29 @@ static void loop_handle_reads_writes(struct mosquitto *context, uint32_t events)
 			){
 
 		do{
-			rc = packet__read(context);
+			switch(context->transport){
+				case mosq_t_tcp:
+				case mosq_t_ws:
+					rc = packet__read(context);
+					break;
+#if defined(WITH_WEBSOCKETS) && WITH_WEBSOCKETS == WS_IS_BUILTIN
+				case mosq_t_http:
+					rc = http__read(context);
+					break;
+#endif
+#if !defined(WITH_WEBSOCKETS) || WITH_WEBSOCKETS == WS_IS_BUILTIN
+				/* Not supported with LWS */
+				case mosq_t_proxy_v2:
+					rc = proxy_v2__read(context);
+					break;
+				case mosq_t_proxy_v1:
+					rc = proxy_v1__read(context);
+					break;
+#endif
+				default:
+					rc = MOSQ_ERR_INVAL;
+					break;
+			}
 			if(rc){
 				do_disconnect(context, rc);
 				return;
