@@ -1,15 +1,15 @@
 /*
-Copyright (c) 2010-2020 Roger Light <roger@atchoo.org>
+Copyright (c) 2010-2021 Roger Light <roger@atchoo.org>
 
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License 2.0
 and Eclipse Distribution License v1.0 which accompany this distribution.
- 
+
 The Eclipse Public License is available at
    https://www.eclipse.org/legal/epl-2.0/
 and the Eclipse Distribution License is available at
   http://www.eclipse.org/org/documents/edl-v10.php.
- 
+
 SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
 
 Contributors:
@@ -26,14 +26,14 @@ Contributors:
 /* DB read/write */
 extern const unsigned char magic[15];
 #define DB_CHUNK_CFG 1
-#define DB_CHUNK_MSG_STORE 2
+#define DB_CHUNK_BASE_MSG 2
 #define DB_CHUNK_CLIENT_MSG 3
 #define DB_CHUNK_RETAIN 4
 #define DB_CHUNK_SUB 5
 #define DB_CHUNK_CLIENT 6
 /* End DB read/write */
 
-#define read_e(f, b, c) if(fread(b, 1, c, f) != c){ goto error; }
+#define read_e(f, b, c) if(fread(b, 1, c, f) != c){ rc = MOSQ_ERR_UNKNOWN; goto error; }
 #define write_e(f, b, c) if(fwrite(b, 1, c, f) != c){ goto error; }
 
 /* COMPATIBILITY NOTES
@@ -49,25 +49,25 @@ extern const unsigned char magic[15];
  * whether what is being added can go in an existing hole in the struct.
  */
 
-struct PF_header{
+struct PF_header {
 	uint32_t chunk;
 	uint32_t length;
 };
 
 
-struct PF_cfg{
+struct PF_cfg {
 	uint64_t last_db_id;
 	uint8_t shutdown;
 	uint8_t dbid_size;
 };
 
-struct PF_client_v5{
+struct PF_client_v5 {
 	int64_t session_expiry_time;
 	uint32_t session_expiry_interval;
 	uint16_t last_mid;
 	uint16_t id_len;
 };
-struct PF_client{
+struct PF_client {
 	/* struct PF_client_v5; */
 	int64_t session_expiry_time;
 	uint32_t session_expiry_interval;
@@ -76,17 +76,17 @@ struct PF_client{
 
 	uint16_t listener_port;
 	uint16_t username_len;
-	/* tail: 4 byte padding, because 64bit member 
+	/* tail: 4 byte padding, because 64bit member
 	 * forces multiple of 8 for struct size */
 };
-struct P_client{
+struct P_client {
 	struct PF_client F;
-	char *client_id;
+	char *clientid;
 	char *username;
 };
 
 
-struct PF_client_msg{
+struct PF_client_msg {
 	dbid_t store_id;
 	uint16_t mid;
 	uint16_t id_len;
@@ -95,14 +95,14 @@ struct PF_client_msg{
 	uint8_t retain_dup;
 	uint8_t direction;
 };
-struct P_client_msg{
+struct P_client_msg {
 	struct PF_client_msg F;
-	char *client_id;
-	mosquitto_property *properties;
+	char *clientid;
+	uint32_t subscription_identifier;
 };
 
 
-struct PF_msg_store{
+struct PF_base_msg {
 	dbid_t store_id;
 	int64_t expiry_time;
 	uint32_t payloadlen;
@@ -114,8 +114,8 @@ struct PF_msg_store{
 	uint8_t qos;
 	uint8_t retain;
 };
-struct P_msg_store{
-	struct PF_msg_store F;
+struct P_base_msg {
+	struct PF_base_msg F;
 	void *payload;
 	struct mosquitto source;
 	char *topic;
@@ -123,24 +123,24 @@ struct P_msg_store{
 };
 
 
-struct PF_sub{
+struct PF_sub {
 	uint32_t identifier;
 	uint16_t id_len;
 	uint16_t topic_len;
 	uint8_t qos;
 	uint8_t options;
 };
-struct P_sub{
+struct P_sub {
 	struct PF_sub F;
-	char *client_id;
+	char *clientid;
 	char *topic;
 };
 
 
-struct PF_retain{
+struct PF_retain {
 	dbid_t store_id;
 };
-struct P_retain{
+struct P_retain {
 	struct PF_retain F;
 };
 
@@ -154,7 +154,7 @@ int persist__chunk_header_read_v234(FILE *db_fptr, uint32_t *chunk, uint32_t *le
 int persist__chunk_cfg_read_v234(FILE *db_fptr, struct PF_cfg *chunk);
 int persist__chunk_client_read_v234(FILE *db_fptr, struct P_client *chunk, uint32_t db_version);
 int persist__chunk_client_msg_read_v234(FILE *db_fptr, struct P_client_msg *chunk);
-int persist__chunk_msg_store_read_v234(FILE *db_fptr, struct P_msg_store *chunk, uint32_t db_version);
+int persist__chunk_base_msg_read_v234(FILE *db_fptr, struct P_base_msg *chunk, uint32_t db_version);
 int persist__chunk_retain_read_v234(FILE *db_fptr, struct P_retain *chunk);
 int persist__chunk_sub_read_v234(FILE *db_fptr, struct P_sub *chunk);
 
@@ -162,14 +162,14 @@ int persist__chunk_header_read_v56(FILE *db_fptr, uint32_t *chunk, uint32_t *len
 int persist__chunk_cfg_read_v56(FILE *db_fptr, struct PF_cfg *chunk);
 int persist__chunk_client_read_v56(FILE *db_fptr, struct P_client *chunk, uint32_t db_version);
 int persist__chunk_client_msg_read_v56(FILE *db_fptr, struct P_client_msg *chunk, uint32_t length);
-int persist__chunk_msg_store_read_v56(FILE *db_fptr, struct P_msg_store *chunk, uint32_t length);
+int persist__chunk_base_msg_read_v56(FILE *db_fptr, struct P_base_msg *chunk, uint32_t length);
 int persist__chunk_retain_read_v56(FILE *db_fptr, struct P_retain *chunk);
 int persist__chunk_sub_read_v56(FILE *db_fptr, struct P_sub *chunk);
 
 int persist__chunk_cfg_write_v6(FILE *db_fptr, struct PF_cfg *chunk);
 int persist__chunk_client_write_v6(FILE *db_fptr, struct P_client *chunk);
 int persist__chunk_client_msg_write_v6(FILE *db_fptr, struct P_client_msg *chunk);
-int persist__chunk_message_store_write_v6(FILE *db_fptr, struct P_msg_store *chunk);
+int persist__chunk_message_store_write_v6(FILE *db_fptr, struct P_base_msg *chunk);
 int persist__chunk_retain_write_v6(FILE *db_fptr, struct P_retain *chunk);
 int persist__chunk_sub_write_v6(FILE *db_fptr, struct P_sub *chunk);
 
