@@ -11,31 +11,31 @@
 
 from mosq_test_helper import *
 
-def do_test():
+def do_test(proto_ver):
     rc = 1
     mid = 53
     keepalive = 60
-    props = mqtt5_props.gen_uint32_prop(mqtt5_props.PROP_SESSION_EXPIRY_INTERVAL, 60)
-    connect_packet = mosq_test.gen_connect("subpub-qos0-test", keepalive=keepalive, proto_ver=5, clean_session=False, properties=props)
-    connack1_packet = mosq_test.gen_connack(rc=0, proto_ver=5)
-    connack2_packet = mosq_test.gen_connack(rc=0, proto_ver=5, flags=1)
+    props = mqtt5_props.gen_uint32_prop(mqtt5_props.SESSION_EXPIRY_INTERVAL, 60)
+    connect_packet = mosq_test.gen_connect("subpub-qos0-test", keepalive=keepalive, proto_ver=proto_ver, clean_session=False, properties=props)
+    connack1_packet = mosq_test.gen_connack(rc=0, proto_ver=proto_ver)
+    connack2_packet = mosq_test.gen_connack(rc=0, proto_ver=proto_ver, flags=1)
 
-    subscribe_packet = mosq_test.gen_subscribe(mid, "subpub/qos1", 1, proto_ver=5)
-    suback_packet = mosq_test.gen_suback(mid, 1, proto_ver=5)
+    subscribe_packet = mosq_test.gen_subscribe(mid, "subpub/qos1", 1, proto_ver=proto_ver)
+    suback_packet = mosq_test.gen_suback(mid, 1, proto_ver=proto_ver)
 
 
 
-    helper_connect = mosq_test.gen_connect("helper", proto_ver=5)
-    helper_connack = mosq_test.gen_connack(rc=0, proto_ver=5)
+    helper_connect = mosq_test.gen_connect("helper", proto_ver=proto_ver)
+    helper_connack = mosq_test.gen_connack(rc=0, proto_ver=proto_ver)
 
     mid=1
-    props = mqtt5_props.gen_uint32_prop(mqtt5_props.PROP_MESSAGE_EXPIRY_INTERVAL, 1)
-    publish1s_packet = mosq_test.gen_publish("subpub/qos1", mid=mid, qos=1, payload="message1", proto_ver=5, properties=props)
+    props = mqtt5_props.gen_uint32_prop(mqtt5_props.MESSAGE_EXPIRY_INTERVAL, 1)
+    publish1s_packet = mosq_test.gen_publish("subpub/qos1", mid=mid, qos=1, payload="message1", proto_ver=proto_ver, properties=props)
     puback1s_packet = mosq_test.gen_puback(mid)
 
     mid=2
-    props = mqtt5_props.gen_uint32_prop(mqtt5_props.PROP_MESSAGE_EXPIRY_INTERVAL, 10)
-    publish2s_packet = mosq_test.gen_publish("subpub/qos1", mid=mid, qos=1, payload="message2", proto_ver=5, properties=props)
+    props = mqtt5_props.gen_uint32_prop(mqtt5_props.MESSAGE_EXPIRY_INTERVAL, 10)
+    publish2s_packet = mosq_test.gen_publish("subpub/qos1", mid=mid, qos=1, payload="message2", proto_ver=proto_ver, properties=props)
     puback2s_packet = mosq_test.gen_puback(mid)
 
 
@@ -52,12 +52,12 @@ def do_test():
         mosq_test.do_send_receive(helper, publish2s_packet, puback2s_packet, "puback 2")
 
         time.sleep(2)
-    
+
         sock = mosq_test.do_client_connect(connect_packet, connack2_packet, timeout=20, port=port)
         packet = sock.recv(len(publish2s_packet))
         for i in range(9, 5, -1):
-            props = mqtt5_props.gen_uint32_prop(mqtt5_props.PROP_MESSAGE_EXPIRY_INTERVAL, i)
-            publish2r_packet = mosq_test.gen_publish("subpub/qos1", mid=2, qos=1, payload="message2", proto_ver=5, properties=props)
+            props = mqtt5_props.gen_uint32_prop(mqtt5_props.MESSAGE_EXPIRY_INTERVAL, i)
+            publish2r_packet = mosq_test.gen_publish("subpub/qos1", mid=2, qos=1, payload="message2", proto_ver=proto_ver, properties=props)
             if packet == publish2r_packet:
                 rc = 0
                 break
@@ -67,7 +67,9 @@ def do_test():
         pass
     finally:
         broker.terminate()
-        broker.wait()
+        if mosq_test.wait_for_subprocess(broker):
+            print("broker not terminated")
+            if rc == 0: rc=1
         (stdo, stde) = broker.communicate()
         if rc:
             print(stde.decode('utf-8'))
@@ -75,5 +77,5 @@ def do_test():
             exit(rc)
 
 
-do_test()
+do_test(proto_ver=5)
 exit(0)
