@@ -5,8 +5,7 @@ from mosq_test_helper import *
 port = mosq_test.get_port()
 
 rc = 1
-keepalive = 60
-connect_packet = mosq_test.gen_connect("test-helper", keepalive=keepalive)
+connect_packet = mosq_test.gen_connect("test-helper")
 connack_packet = mosq_test.gen_connack(rc=0)
 
 publish_packet = mosq_test.gen_publish("bridge/ssl/test", qos=0, payload="message")
