@@ -8,13 +8,21 @@
 static int run = -1;
 static int sent_mid;
 
-void on_log(struct mosquitto *mosq, void *obj, int level, const char *str)
+
+static void on_log(struct mosquitto *mosq, void *obj, int level, const char *str)
 {
+	(void)mosq;
+	(void)obj;
+	(void)level;
+
 	printf("%s\n", str);
 }
 
-void on_connect(struct mosquitto *mosq, void *obj, int rc)
+
+static void on_connect(struct mosquitto *mosq, void *obj, int rc)
 {
+	(void)obj;
+
 	if(rc){
 		exit(1);
 	}else{
@@ -22,8 +30,11 @@ void on_connect(struct mosquitto *mosq, void *obj, int rc)
 	}
 }
 
-void on_publish(struct mosquitto *mosq, void *obj, int mid)
+
+static void on_publish(struct mosquitto *mosq, void *obj, int mid)
 {
+	(void)obj;
+
 	if(mid == sent_mid){
 		mosquitto_disconnect(mosq);
 		run = 0;
@@ -32,17 +43,25 @@ void on_publish(struct mosquitto *mosq, void *obj, int mid)
 	}
 }
 
-void on_disconnect(struct mosquitto *mosq, void *obj, int rc)
+
+static void on_disconnect(struct mosquitto *mosq, void *obj, int rc)
 {
+	(void)mosq;
+	(void)obj;
+
 	run = rc;
 }
 
+
 int main(int argc, char *argv[])
 {
 	int rc;
 	struct mosquitto *mosq;
 	int port;
 
+	if(argc < 2){
+		return 1;
+	}
 	port = atoi(argv[1]);
 
 	mosquitto_lib_init();
@@ -55,12 +74,15 @@ int main(int argc, char *argv[])
 	mosquitto_log_callback_set(mosq, on_log);
 
 	rc = mosquitto_connect(mosq, "localhost", port, 60);
-	if(rc) return rc;
+	if(rc){
+		return rc;
+	}
 
 	while(run == -1){
 		mosquitto_loop(mosq, -1, 1);
 	}
 
+	mosquitto_destroy(mosq);
 	mosquitto_lib_cleanup();
 	return run;
 }
