@@ -1,45 +1,55 @@
-#include <mosquittopp.h>
+#include <cassert>
+#include <mosquitto/libmosquittopp.h>
 
 static int run = -1;
 
 class mosquittopp_test : public mosqpp::mosquittopp
 {
-	public:
-		mosquittopp_test(const char *id);
+public:
+	mosquittopp_test(const char *id);
 
-		void on_connect(int rc);
-		void on_disconnect(int rc);
-		void on_subscribe(int mid, int qos_count, const int *granted_qos);
+	void on_connect(int rc);
+	void on_disconnect(int rc);
+	void on_subscribe(int mid, int qos_count, const int *granted_qos);
 };
 
 mosquittopp_test::mosquittopp_test(const char *id) : mosqpp::mosquittopp(id)
 {
 }
 
+
 void mosquittopp_test::on_connect(int rc)
 {
 	if(rc){
 		exit(1);
 	}else{
-		subscribe(NULL, "qos2/test", 2);
+		subscribe_v5(NULL, "qos2/test", 2, 2, NULL);
 	}
 }
 
+
 void mosquittopp_test::on_disconnect(int rc)
 {
 	run = rc;
 }
 
+
 void mosquittopp_test::on_subscribe(int mid, int qos_count, const int *granted_qos)
 {
+	assert(mid == 1);
+	assert(qos_count == 1);
+	assert(granted_qos[0] == 2);
 	disconnect();
 }
 
 
 int main(int argc, char *argv[])
 {
-	struct mosquittopp_test *mosq;
+	mosquittopp_test *mosq;
 
+	if(argc != 2){
+		return 1;
+	}
 	int port = atoi(argv[1]);
 
 	mosqpp::lib_init();
@@ -51,7 +61,6 @@ int main(int argc, char *argv[])
 	while(run == -1){
 		mosq->loop();
 	}
-	delete mosq;
 
 	delete mosq;
 	mosqpp::lib_cleanup();
