@@ -1,3 +1,6 @@
+#ifndef MSGSPS_COMMON_H
+#define MSGSPS_COMMON_H
+
 #define HOST "127.0.0.1"
 #define PORT 1883
 
@@ -5,3 +8,5 @@
 #define SUB_QOS 1
 #define MESSAGE_SIZE 1024L
 
+#endif
+
